/*  fits.c  for fitsjd */
#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include <fitsio.h>
#include "functs.h"
#include "consts.h"

/* 
    The datatype parameter can have one of the following symbolic constant 
    values: TSTRING, TLOGICAL, TBYTE, TSHORT, TINT, TLONG, TFLOAT, TDOUBLE.
*/
int
read_fits(char *name,  FITSFILE *f, char **errors)
     
{
  int status = 0;
  char error[81], comment[81], temp[81];
  int verbose = 0;
  int errnum = 0;
  int nfound;

  /*  open the file  */
  fits_open_file(&f->fptr, name, READONLY, &status);
  if (status != 0)  {
    do  {
      fits_read_errmsg(error);
      errors[errnum++] = strdup(error);
    } while(strlen(error) > 0 && errnum < ERROR_MAX - 1);
    if (errnum == ERROR_MAX - 1) {
      errors[errnum] = "And there may be other problems, too";
    }
    return errnum;
  }
  /*  Check if the file is simple fits file  */
  fits_read_key_log(f->fptr, "SIMPLE", &f->simple, comment, &status);
  if (verbose)  {
    printf("SIMPLE=%c\n", f->simple ? 'T' : 'F');
  }
  if (!f->simple) {
    errors[errnum] = "I am not supposed to handle nonsimple files";
    return errnum;
  }
  /*  Read number of axis  */
  fits_read_key_lng(f->fptr, "NAXIS", &f->naxis, comment, &status);
  if (verbose)  {
    printf("NAXIS=%ld\n", f->naxis);
  }
  if (f->naxis != 2)  {
    errors[errnum++] = "At the moment, I can handle only files with 2 axis";
    sprintf(temp, "File %s has %ld axis\n", name, f->naxis);
    errors[errnum++] = strdup(temp);
    do  {
      fits_read_errmsg(error);
      errors[errnum++] = strdup(error);
    } while(strlen(error) > 0 && errnum < ERROR_MAX - 1);
    if (errnum == ERROR_MAX - 1) {
      errors[errnum] = "And there may be other problems, too.";
    }
    return errnum;
  }
  /*  Read dimensions at once  */
  fits_read_keys_lng(f->fptr, "NAXIS", 1, f->naxis, f->nax, &nfound, &status);
  if (verbose)  {
    int i;
    for (i = 0; i < f->naxis; i++)  {
      printf("NAXIS%d=%ld\n", i + 1, f->nax[i]);
    }
  }

  /*============================================================*/
  /*  Try to read WCS coordinates                               */
  /*============================================================*/
  status = 0;
  if (verbose)  {
    printf("Reading WCS\n");
  }
  fits_read_img_coord(f->fptr,  &f->crval1, &f->crval2, 
       &f->crpix1, &f->crpix2, &f->cdelt1, &f->cdelt2,
       &f->crot, f->ctype, &status);
  /*  Check if everything about WCS was read properly  */
  f->wcs = (status == 0);
  if (f->wcs) {
    /*  if ctype is empty, assume -TAN  */
    if (strlen(f->ctype) == 0) {
      strcpy(f->ctype, "-TAN");
    }
    /*  Read CTYPE1 keyword  */
    status = 0;
    fits_read_keyword(f->fptr, "CTYPE1", f->ctype1, comment, &status);
    /*  if CTYPE1 is DEC, than exchange the crval1 and crval2 values  */
    if (strncmp(f->ctype1 + 1, "DEC", 3) == 0) {
      double t;
      t = f->crval1;
      f->crval1 = f->crval2;
      f->crval2 = t;
    }
  }
  status = 0;
  /*  Check if WCS values were already computed using a star match with a catalog */
  f->matched = 0;
  fits_read_key_log(f->fptr, "MATCHED", &f->matched, comment, &status);
  if (verbose)  {
    printf("MATCHED=%c\n", f->matched ? 'T' : 'F');
  }
  /*============================================================*/
  /*  Get date and time of observation and exposure time        */
  /*============================================================*/
  status = 0;
  fits_read_keyword(f->fptr, "DATE-OBS", f->date_obs, comment, &status);
  f->is_date_obs = (status == 0);   /* Flag success on reading date  */
  status = 0;
  fits_read_key(f->fptr, TFLOAT, "EXPOSURE", &(f->exposure), comment, &status);
  f->is_exposure = (status == 0);   /* Flag success on reading exposure time  */
  if (status != 0) {
    /*  Try again with a nonstandard keyword  */
    status = 0;
    fits_read_key(f->fptr, TFLOAT, "EXPTIME", &(f->exposure), comment, &status);
    f->is_exposure = (status == 0);   /* Flag success on reading exposure time  */
  }
  status = 0;
  f->ut_start[0] = '\0';
  fits_read_keyword(f->fptr, "UT", f->ut_start, comment, &status);
  f->is_ut_start = (status == 0);   /* Flag success on reading start-of-exposure time  */
  if (!f->is_ut_start) {
      status = 0;
      fits_read_keyword(f->fptr, "UT-START", f->ut_start, comment, &status);
      f->is_ut_start = (status == 0);   /* Flag success on reading start-of-exposure time  */
  }
  status = 0;
  fits_read_keyword(f->fptr, "UT-CENT", f->ut_cent, comment, &status);
  f->is_ut_cent = (status == 0);   /* Flag success on reading mid-exposure time  */
  status = 0;
  fits_read_keyword(f->fptr, "UT-END", f->ut_end, comment, &status);
  f->is_ut_end = (status == 0);   /* Flag success on reading end-of-exposure time  */
  status = 0;
  fits_read_keyword(f->fptr, "TIME-BEG", f->time_beg, comment, &status);
  f->is_time_beg = (status == 0);   /* Flag success on reading start-of-exposure time  */
  status = 0;
  fits_read_keyword(f->fptr, "TIME-END", f->time_end, comment, &status);
  f->is_time_end = (status == 0);   /* Flag success on reading start-of-exposure time  */
  status = 0;
  /*  Close the file  */
  fits_close_file(f->fptr, &status);
  if (status != 0) {
    do  {
      fits_read_errmsg(error);
      errors[errnum++] = strdup(error);
    } while(strlen(error) > 0 && errnum < ERROR_MAX - 1);
    if (errnum == ERROR_MAX - 1) {
      errors[errnum] = "And there may be other problems, too.";
    }
  }
  if (verbose)  {
    printf("End of read_fits\n");
  }
  return errnum;
}



